package io.tswf.minecraft.blockventure.core.transmutation

import groovy.transform.CompileStatic
import io.tswf.groovy.bettergroovy.asm.transmutation.annotation.AtFieldCall
import io.tswf.groovy.bettergroovy.asm.transmutation.annotation.ClassReference
import io.tswf.groovy.bettergroovy.asm.transmutation.annotation.MethodDescription
import io.tswf.groovy.bettergroovy.asm.transmutation.annotation.Offset
import io.tswf.groovy.bettergroovy.asm.transmutation.annotation.Self
import io.tswf.groovy.bettergroovy.asm.transmutation.annotation.Transmute
import io.tswf.groovy.bettergroovy.asm.transmutation.injection.contexts.InjectionOffset
import io.tswf.groovy.bettergroovy.asm.transmutation.injection.points.FieldCallType
import io.tswf.groovy.bettergroovy.asm.transmutation.meta.MissingTargetStrategy
import io.tswf.minecraft.blockventure.core.api.registry.BlockEntityRegistry
import io.tswf.minecraft.blockventure.core.view.ViewBlockEntity
import net.minecraft.block.BlockState
import net.minecraft.block.entity.BlockEntity
import net.minecraft.block.entity.BlockEntityType
import net.minecraft.util.math.BlockPos

@CompileStatic
class TransmutationBlockEntity {

    @Transmute(
            targetClass = @ClassReference(BlockEntity),
            targetMethod = @MethodDescription(
                    name = "<init>",
                    parameterTypeClasses = [ BlockEntityType, BlockPos, BlockState ],
                    returnType = @ClassReference(descriptor = "V")
            ),
            onMissing = MissingTargetStrategy.THROW
    )
    @AtFieldCall(
            callType = FieldCallType.SET,
            name = "type",
            type = @ClassReference(BlockEntityType),
            owner = @ClassReference(BlockEntity)
    )
    @Offset(InjectionOffset.AFTER)
    static void fetchBlockEntityTypeIfNull(BlockEntityType entityType, BlockPos blockPos, BlockState blockState, @Self BlockEntity self) {
        def viewBlockEntity = (ViewBlockEntity) self
        if (entityType == null) {
            viewBlockEntity.type = BlockEntityRegistry.findBlockEntityTypeByClass(self.class)
        }
    }

}
